@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -30,7 +32,7 @@ class diagmat_proxy_default
   inline
   diagmat_proxy_default(const T1& X)
     : P       ( X )
-    , P_is_vec( (resolves_to_vector<T1>::value) || (P.get_n_rows() == 1) || (P.get_n_cols() == 1) )
+    , P_is_vec( (resolves_to_vector<T1>::yes) || (P.get_n_rows() == 1) || (P.get_n_cols() == 1) )
     , P_is_col( T1::is_col || (P.get_n_cols() == 1) )
     , n_rows  ( P_is_vec ? P.get_n_elem() : P.get_n_rows() )
     , n_cols  ( P_is_vec ? P.get_n_elem() : P.get_n_cols() )
@@ -90,7 +92,7 @@ class diagmat_proxy_default
     }
   
   
-  arma_inline bool is_alias(const Mat<elem_type>&) const { return false; }
+  constexpr bool is_alias(const Mat<elem_type>&) const { return false; }
   
   const Proxy<T1> P;
   const bool      P_is_vec;
@@ -143,6 +145,12 @@ class diagmat_proxy_fixed
   
   const T1& P;
   
+  //// this may require T1::n_elem etc to be declared as static constexpr inline variables (C++17)
+  //// see also the notes in Mat::fixed
+  // static constexpr bool  P_is_vec = (T1::n_rows == 1) || (T1::n_cols == 1);
+  // static constexpr uword n_rows   = P_is_vec ? T1::n_elem : T1::n_rows;
+  // static constexpr uword n_cols   = P_is_vec ? T1::n_elem : T1::n_cols;
+  
   static const bool  P_is_vec = (T1::n_rows == 1) || (T1::n_cols == 1);
   static const uword n_rows   = P_is_vec ? T1::n_elem : T1::n_rows;
   static const uword n_cols   = P_is_vec ? T1::n_elem : T1::n_cols;
@@ -161,11 +169,11 @@ struct diagmat_proxy_redirect<T1, true>  { typedef diagmat_proxy_fixed<T1>   res
 
 
 template<typename T1>
-class diagmat_proxy : public diagmat_proxy_redirect<T1, is_Mat_fixed<T1>::value >::result
+class diagmat_proxy : public diagmat_proxy_redirect<T1, is_Mat_fixed<T1>::value>::result
   {
   public:
   inline diagmat_proxy(const T1& X)
-    : diagmat_proxy_redirect< T1, is_Mat_fixed<T1>::value >::result(X)
+    : diagmat_proxy_redirect<T1, is_Mat_fixed<T1>::value>::result(X)
     {
     }
   };
@@ -226,7 +234,7 @@ class diagmat_proxy< Row<eT> >
   
   arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&P)); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const Row<eT>& P;
   const uword    n_rows;
@@ -258,7 +266,7 @@ class diagmat_proxy< Col<eT> >
   
   arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&P)); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const Col<eT>& P;
   const uword    n_rows;
@@ -290,7 +298,7 @@ class diagmat_proxy< subview_row<eT> >
   
   arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&(P.m))); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const subview_row<eT>& P;
   const uword            n_rows;
@@ -322,7 +330,7 @@ class diagmat_proxy< subview_col<eT> >
   
   arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&(P.m))); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const subview_col<eT>& P;
   const uword            n_rows;
@@ -331,6 +339,39 @@ class diagmat_proxy< subview_col<eT> >
 
 
 
+template<typename T1, typename T2>
+class diagmat_proxy< Glue<T1,T2,glue_times> >
+  {
+  public:
+  
+  typedef typename T1::elem_type                   elem_type;
+  typedef typename get_pod_type<elem_type>::result pod_type;
+  
+  inline
+  diagmat_proxy(const Glue<T1,T2,glue_times>& X)
+    {
+    op_diagmat::apply_times(P, X.A, X.B);
+    
+    n_rows = P.n_rows;
+    n_cols = P.n_cols;
+    
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_inline elem_type operator[] (const uword i)                    const { return P.at(i,i);                                   }
+  arma_inline elem_type at         (const uword row, const uword col) const { return (row == col) ? P.at(row,row) : elem_type(0); }
+  
+  constexpr bool is_alias(const Mat<elem_type>&) const { return false; }
+  
+  static constexpr bool P_is_vec = false;
+  
+  Mat<elem_type> P;
+  uword          n_rows;
+  uword          n_cols;
+  };
+
+
+
 //
 //
 //
@@ -348,7 +389,7 @@ class diagmat_proxy_check_default
   inline
   diagmat_proxy_check_default(const T1& X, const Mat<typename T1::elem_type>&)
     : P(X)
-    , P_is_vec( (resolves_to_vector<T1>::value) || (P.n_rows == 1) || (P.n_cols == 1) )
+    , P_is_vec( (resolves_to_vector<T1>::yes) || (P.n_rows == 1) || (P.n_cols == 1) )
     , n_rows( P_is_vec ? P.n_elem : P.n_rows )
     , n_cols( P_is_vec ? P.n_elem : P.n_cols )
     {
@@ -388,9 +429,9 @@ class diagmat_proxy_check_fixed
   
   const Mat<eT> P;  // TODO: why not just store X directly as T1& ?  test with fixed size vectors and matrices
   
-  static const bool  P_is_vec = (T1::n_rows == 1) || (T1::n_cols == 1);
-  static const uword n_rows   = P_is_vec ? T1::n_elem : T1::n_rows;
-  static const uword n_cols   = P_is_vec ? T1::n_elem : T1::n_cols;
+  static constexpr bool  P_is_vec = (T1::n_rows == 1) || (T1::n_cols == 1);
+  static constexpr uword n_rows   = P_is_vec ? T1::n_elem : T1::n_rows;
+  static constexpr uword n_cols   = P_is_vec ? T1::n_elem : T1::n_cols;
   };
 
 
@@ -406,11 +447,11 @@ struct diagmat_proxy_check_redirect<T1, true>  { typedef diagmat_proxy_check_fix
 
 
 template<typename T1>
-class diagmat_proxy_check : public diagmat_proxy_check_redirect<T1, is_Mat_fixed<T1>::value >::result
+class diagmat_proxy_check : public diagmat_proxy_check_redirect<T1, is_Mat_fixed<T1>::value>::result
   {
   public:
   inline diagmat_proxy_check(const T1& X, const Mat<typename T1::elem_type>& out)
-    : diagmat_proxy_check_redirect< T1, is_Mat_fixed<T1>::value >::result(X, out)
+    : diagmat_proxy_check_redirect<T1, is_Mat_fixed<T1>::value>::result(X, out)
     {
     }
   };
@@ -480,7 +521,7 @@ class diagmat_proxy_check< Row<eT> >
   arma_inline elem_type operator[] (const uword i)                    const { return P[i];                                 }
   arma_inline elem_type at         (const uword row, const uword col) const { return (row == col) ? P[row] : elem_type(0); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const Row<eT>* P_local;
   const Row<eT>& P;
@@ -516,7 +557,7 @@ class diagmat_proxy_check< Col<eT> >
   arma_inline elem_type operator[] (const uword i)                    const { return P[i];                                 }
   arma_inline elem_type at         (const uword row, const uword col) const { return (row == col) ? P[row] : elem_type(0); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const Col<eT>* P_local;
   const Col<eT>& P;
@@ -546,7 +587,7 @@ class diagmat_proxy_check< subview_row<eT> >
   arma_inline elem_type operator[] (const uword i)                    const { return P[i];                                 }
   arma_inline elem_type at         (const uword row, const uword col) const { return (row == col) ? P[row] : elem_type(0); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const Row<eT> P;
   const uword   n_rows;
@@ -575,7 +616,7 @@ class diagmat_proxy_check< subview_col<eT> >
   arma_inline elem_type operator[] (const uword i)                    const { return P[i];                                 }
   arma_inline elem_type at         (const uword row, const uword col) const { return (row == col) ? P[row] : elem_type(0); }
   
-  static const bool P_is_vec = true;
+  static constexpr bool P_is_vec = true;
   
   const Col<eT> P;
   const uword   n_rows;
@@ -584,4 +625,35 @@ class diagmat_proxy_check< subview_col<eT> >
 
 
 
+template<typename T1, typename T2>
+class diagmat_proxy_check< Glue<T1,T2,glue_times> >
+  {
+  public:
+  
+  typedef typename T1::elem_type                   elem_type;
+  typedef typename get_pod_type<elem_type>::result pod_type;
+  
+  inline
+  diagmat_proxy_check(const Glue<T1,T2,glue_times>& X, const Mat<elem_type>&)
+    {
+    op_diagmat::apply_times(P, X.A, X.B);
+    
+    n_rows = P.n_rows;
+    n_cols = P.n_cols;
+    
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_inline elem_type operator[] (const uword i)                    const { return P.at(i,i);                                   }
+  arma_inline elem_type at         (const uword row, const uword col) const { return (row == col) ? P.at(row,row) : elem_type(0); }
+  
+  static constexpr bool P_is_vec = false;
+  
+  Mat<elem_type> P;
+  uword          n_rows;
+  uword          n_cols;
+  };
+
+
+
 //! @}
